/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: ...
   co-programmer: Patrick Boenzli
*/

//#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_WORLD_ENTITY

#include "particle_emitter.h"

#include "particle_system.h"

using namespace std;


/**
   \brief standard constructor
*/
ParticleEmitter::ParticleEmitter(const Vector& direction, float angle, float emissionRate, 
		  float velocity)
{
   this->setClassName ("ParticleEmitter");
   this->direction = direction;
   this->setSpread(angle);
   this->emissionRate = emissionRate;
   this->velocity = velocity;

   this->saveTime = 0.0;
}



/**
   \brief standard destructor

*/
ParticleEmitter::~ParticleEmitter () 
{}


/**
   \brief this start the emitter
*/
void ParticleEmitter::start() {}


/**
   \brief this stops the emitter
*/
void ParticleEmitter::stop() {}




/* these are Animation interfaces: so you can change spec values as you want */

/**
   \brief set the emission rate
   \param sets the number of particles emitted per second
   \param random A random emissionRate, the +- randomness of this option

   if you want to change the value of this variable during emission time (to make it more dynamic)
   you may want to use the animation class
*/
void ParticleEmitter::setEmissionRate(float emissionRate)
{}

/**
   \brief set the angle of the emitter
   \param angle around the direction in which there are particles to be emitted
   \param randomAngle A random spread-angle, the +- randomness of this option

   if you want to change the value of this variable during emission time (to make it more dynamic)
   you may want to use the animation class
*/
void ParticleEmitter::setSpread(float angle, float randomAngle)
{}




/**
   \brief sets the velocity of all particles emitted
   \param velocity The starting velocity of the emitted particles
   \param random A random starting velocity, the +- randomness of this option

   if you want to change the value of this variable during emission time (to make it more dynamic)
   you may want to use the animation class
*/
void ParticleEmitter::setVelocity(float velocity, float random)
{}

/**
   \brief this set the time to life of a particle, after which it will die
   \param the time to live in seconds

   if you want to change the value of this variable during emission time (to make it more dynamic)
   you may want to use the animation class
*/

void ParticleEmitter::tick(float dt, ParticleSystem* system)
{
  // saving the time
  float countF = (dt+this->saveTime) / emissionRate;
  int count = (int)round(countF);
  this->saveTime = countF - (float)count;

  for (int i = 0; i <= count; i++)
    // emmits from EMITTER_DOT,
    system->addParticle(this->getAbsCoor(), direction * velocity );
}
